@@ -2,7 +2,7 @@ from .lexer import Lexer
 from .lexer.parser import *
 from .lexer.types import *
 from .lexer.result import *
-from .lexer.ast import *
+from .lexer.avo_ast import *
 
 
-version = 'v0.2.5'
+version = 'v0.3.5'
