@@ -2,7 +2,7 @@
 from functools import reduce
 from pprint import pprint
 
-from .ast import *
+from . import avo_ast
 from .combinator import *
 from .types import Token, TokenType
 
@@ -33,7 +33,7 @@ id_tag = Tag(TokenType.ID)
 num = Tag(TokenType.INT) ^ (lambda x: int(x))
 float_num = Tag(TokenType.FLOAT) ^ (lambda x: float(x))
 boolean = Tag(TokenType.BOOL) ^ process_boolean
-string = Tag(TokenType.STRING) ^ (lambda x: StringAST(x[1:-1]))
+string = Tag(TokenType.STRING) ^ (lambda x: avo_ast.StringAST(x[1:-1]))
 a_expr_precedence_levels = [
     ['*', '/'],
     ['+', '-', '%'],
@@ -54,14 +54,14 @@ b_expr_precedence_levels = [
 def array_expr():
     def process(p):
         (_, data), _ = p
-        return ArrayAST(data)
+        return avo_ast.ArrayAST(data)
     return keyword('[') + Opt(Rep(Lazy(expr) + Opt(keyword(',')))) + keyword(']') ^ process
 
 
 def if_else_expr():
     def process(p):
         (((body, op1), condition), op2), else_body = p
-        return TernaryOpAST(body, op1, condition, op2, else_body)
+        return avo_ast.TernaryOpAST(body, op1, condition, op2, else_body)
     return (
             Lazy(expr) + Alt(keyword('if'), operator('?')) + Lazy(expr) +
             Alt(keyword('else'), operator(':')) + Lazy(expression)
@@ -74,10 +74,10 @@ def lambda_stmt():
         arguments = []
         for arg in args:
             if arg.value[0][1] is None:
-                arguments.append(ArgumentAST(arg.value[0][0], None))
+                arguments.append(avo_ast.ArgumentAST(arg.value[0][0], None))
             else:
-                arguments.append(ArgumentAST(arg.value[0][0], arg.value[0][1][1]))
-        return LambdaStmt(arguments, statements)
+                arguments.append(avo_ast.ArgumentAST(arg.value[0][0], arg.value[0][1][1]))
+        return avo_ast.LambdaStmt(arguments, statements)
 
     return (
             keyword('(') + Rep(id_tag + Opt(operator('=') + Lazy(expression)) + Opt(keyword(','))) +
@@ -88,7 +88,7 @@ def lambda_stmt():
 def module_obj_expr():
     def process(p):
         (module, _), var = p
-        return ModuleCallAST(module, var)
+        return avo_ast.ModuleCallAST(module, var)
     return id_tag + operator('.') + id_tag ^ process
 
 
@@ -99,10 +99,10 @@ def id_or_module():
 def a_expr_value():
     return (
             Lazy(built_in_func_stmt) |
-            (num ^ (lambda x: IntAST(x))) |
-            (float_num ^ (lambda x: FloatAST(x))) |
-            (id_tag ^ (lambda x: VarAST(x))) |
-            (boolean ^ (lambda x: BoolAST(x))) |
+            (num ^ (lambda x: avo_ast.IntAST(x))) |
+            (float_num ^ (lambda x: avo_ast.FloatAST(x))) |
+            (id_tag ^ (lambda x: avo_ast.VarAST(x))) |
+            (boolean ^ (lambda x: avo_ast.BoolAST(x))) |
             string
     )
 
@@ -131,7 +131,7 @@ def a_expr_term():
 
 
 def process_binop(op):
-    return lambda l, r: BinOpAST(op, l, r)
+    return lambda l, r: avo_ast.BinOpAST(op, l, r)
 
 
 def any_op_in_list(ops):
@@ -160,7 +160,7 @@ def a_expr():
 # --== Boolean conditions ==-- #
 def process_relop(p):
     (l, op), r = p
-    return RelativeOp(op, l, r)
+    return avo_ast.RelativeOp(op, l, r)
 
 
 def b_expr_relop():
@@ -170,7 +170,7 @@ def b_expr_relop():
 
 
 def b_expr_not():
-    return (keyword('not') + Lazy(b_expr_term)) ^ (lambda p: NotOp(p[1]))
+    return (keyword('not') + Lazy(b_expr_term)) ^ (lambda p: avo_ast.NotOp(p[1]))
 
 
 def b_expr_group():
@@ -178,17 +178,17 @@ def b_expr_group():
 
 
 def b_expr_term():
-    return b_expr_group() | b_expr_not() | b_expr_relop() | (boolean ^ (lambda x: BoolAST(x)))
+    return b_expr_group() | b_expr_not() | b_expr_relop() | (boolean ^ (lambda x: avo_ast.BoolAST(x)))
 
 
 def process_logic(op):
     match op:
         case 'and' | '&&':
-            return lambda l, r: AndOp(l, r)
+            return lambda l, r: avo_ast.AndOp(l, r)
         case 'or' | '||':
-            return lambda l, r: OrOp(l, r)
+            return lambda l, r: avo_ast.OrOp(l, r)
         case 'in':
-            return lambda l, r: InOp(l, r)
+            return lambda l, r: avo_ast.InOp(l, r)
         case _:
             raise RuntimeError(f'unknown logic operator: {op}')
 
@@ -204,7 +204,7 @@ def brace_expr():
         for i in v_arr:
             (_, i), _ = i.value
             arr.append(i)
-        return BraceAST(obj, [v] + arr)
+        return avo_ast.BraceAST(obj, [v] + arr)
     return (
             (Lazy(array_expr) | Lazy(call_stmt) | string | id_or_module()) + keyword('[') +
             Lazy(expr) + keyword(']') + Rep(keyword('[') + Lazy(expr) + keyword(']'))
@@ -223,7 +223,7 @@ def class_property_stmt():
         ((is_super, name), _), var = p
         if is_super is not None:
             is_super = True
-        return ClassPropAST(name, var, is_super)
+        return avo_ast.ClassPropAST(name, var, is_super)
     return Opt(keyword('super')) + Alt(id_tag, keyword('this')) + operator('::') + id_tag ^ process
 
 
@@ -235,21 +235,21 @@ def expression():
 def assign_stmt():
     def process(p):
         ((_, name), _), e = p
-        return AssignStmt(name, e, False, True)
+        return avo_ast.AssignStmt(name, e, False, True)
     return (keyword('var') + id_tag + operator('=') + expression()) ^ process
 
 
 def assign_const_stmt():
     def process(p):
         ((_, name), _), e = p
-        return AssignStmt(name, e, True, True)
+        return avo_ast.AssignStmt(name, e, True, True)
     return (keyword('const') + id_tag + operator('=') + expression()) ^ process
 
 
 def reassign_stmt():
     def process(p):
         (name, op), e = p
-        return AssignStmt(name, e, False, False, op)
+        return avo_ast.AssignStmt(name, e, False, False, op)
     return ((brace_expr() | id_or_module()) + any_op_in_list(assign_operators) + expression()) ^ process
 
 
@@ -258,13 +258,13 @@ def unary_op_stmt():
         sym, name = p
         if sym not in unary_operators:
             name, sym = sym, name
-        return UnaryOpAST(sym, VarAST(name))
+        return avo_ast.UnaryOpAST(sym, avo_ast.VarAST(name))
     return Alt(any_op_in_list(unary_operators) + id_or_module(), id_or_module() + any_op_in_list(unary_operators)) ^ process
 
 
 def stmt_list():
     def process(rep):
-        return StmtList(rep)
+        return avo_ast.StmtList(rep)
     return Rep(Lazy(stmt) + Opt(keyword(';')) ^ (lambda x: x[0])) ^ process
 
 
@@ -284,7 +284,7 @@ def if_stmt():
             false_body = None
         if elif_array:
             elif_array = [i.value for i in elif_array]
-        return IfStmt(condition, body, elif_array, false_body)
+        return avo_ast.IfStmt(condition, body, elif_array, false_body)
     result = keyword('if') + Exp(b_expr(), None) + keyword('{') + Opt(Lazy(stmt_list)) + keyword('}')
     result += Opt(
         Rep(
@@ -300,23 +300,23 @@ def if_stmt():
 def while_stmt():
     def process(p):
         (((_, condition), _), body), _ = p
-        return WhileStmt(condition, body)
+        return avo_ast.WhileStmt(condition, body)
     result = keyword('while') + Exp(b_expr(), None) + keyword('{') + Opt(Lazy(stmt_list)) + keyword('}')
     return result ^ process
 
 
 def break_stmt():
-    return keyword('break') ^ (lambda x: BreakStmt())
+    return keyword('break') ^ (lambda x: avo_ast.BreakStmt())
 
 
 def continue_stmt():
-    return keyword('continue') ^ (lambda x: ContinueStmt())
+    return keyword('continue') ^ (lambda x: avo_ast.ContinueStmt())
 
 
 def echo_stmt():
     def process(p):
         (_, data), _ = p
-        return EchoStmt(data)
+        return avo_ast.EchoStmt(data)
     return (
             keyword('echo') + keyword('(') +
             Opt(
@@ -330,7 +330,7 @@ def echo_stmt():
 def read_stmt():
     def process(p):
         _, text = p
-        return ReadStmt(text)
+        return avo_ast.ReadStmt(text)
     return keyword('read') + expression() ^ process
 
 
@@ -340,7 +340,7 @@ def built_in_func_stmt():
         args = []
         for a in arg:
             args.append(a.value[0])
-        return BuiltInFuncStmt(name, args)
+        return avo_ast.BuiltInFuncStmt(name, args)
     return (
             any_builtin_in_list(builtins) + keyword('(') +
             Rep(Lazy(expression) + Opt(keyword(','))) + keyword(')')
@@ -353,10 +353,10 @@ def func_stmt():
         arguments = []
         for arg in args:
             if arg.value[0][1] is None:
-                arguments.append(ArgumentAST(arg.value[0][0], None))
+                arguments.append(avo_ast.ArgumentAST(arg.value[0][0], None))
             else:
-                arguments.append(ArgumentAST(arg.value[0][0], arg.value[0][1][1]))
-        return FuncStmt(func_name, arguments, statements)
+                arguments.append(avo_ast.ArgumentAST(arg.value[0][0], arg.value[0][1][1]))
+        return avo_ast.FuncStmt(func_name, arguments, statements)
     return (
             keyword('func') + id_tag + keyword('(') +
             Rep(id_tag + Opt(operator('=') + Lazy(expression)) + Opt(keyword(','))) +
@@ -370,10 +370,10 @@ def interface_func_stmt():
         arguments = []
         for arg in args:
             if arg.value[0][1] is None:
-                arguments.append(ArgumentAST(arg.value[0][0], None))
+                arguments.append(avo_ast.ArgumentAST(arg.value[0][0], None))
             else:
-                arguments.append(ArgumentAST(arg.value[0][0], arg.value[0][1][1]))
-        return FuncStmt(func_name, arguments, StmtList([]))
+                arguments.append(avo_ast.ArgumentAST(arg.value[0][0], arg.value[0][1][1]))
+        return avo_ast.FuncStmt(func_name, arguments, avo_ast.StmtList([]))
     return (
             keyword('func') + id_tag + keyword('(') +
             Rep(id_tag + Opt(operator('=') + Lazy(expression)) + Opt(keyword(','))) +
@@ -383,32 +383,34 @@ def interface_func_stmt():
 
 def call_stmt():
     def process(p):
-        ((func_name, _), args), _ = p
+        (((func_name, _), args), _), l = p
         arguments = []
         for arg in args:
             if arg.value[0][0] is None:
-                arguments.append(ArgumentAST(None, arg.value[0][1]))
+                arguments.append(avo_ast.ArgumentAST(None, arg.value[0][1]))
             else:
-                arguments.append(ArgumentAST(arg.value[0][0][0], arg.value[0][1]))
-        return CallStmt(func_name, arguments)
+                arguments.append(avo_ast.ArgumentAST(arg.value[0][0][0], arg.value[0][1]))
+        if l:
+            arguments.append(avo_ast.ArgumentAST(None, avo_ast.LambdaStmt([], l[0][1])))
+        return avo_ast.CallStmt(func_name, arguments)
     return (
         id_or_module() + keyword('(') +
         Rep(Opt(id_tag + operator('=')) + expression() + Opt(keyword(','))) +
-        keyword(')')
+        keyword(')') + Opt(keyword('{') + Opt(Lazy(stmt_list)) + keyword('}'))
     ) ^ process
 
 
 def return_stmt():
     def process(p):
         _, return_value = p
-        return ReturnStmt(return_value)
+        return avo_ast.ReturnStmt(return_value)
     return keyword('return') + Opt(expression()) ^ process
 
 
 def for_stmt():
     def process(p):
         (((((((_, var), _), cond), _), action), _), body), _ = p
-        return ForStmt(var, cond, action, body)
+        return avo_ast.ForStmt(var, cond, action, body)
     return (
             keyword('for') + Lazy(assign_stmt) + keyword(';') +
             Exp(b_expr(), None) + keyword(';') +
@@ -419,7 +421,7 @@ def for_stmt():
 def foreach_stmt():
     def process(p):
         (((((_, var), _), val), _), body), _ = p
-        return ForStmt(var, val, body, None)
+        return avo_ast.ForStmt(var, val, body, None)
     return (
             keyword('for') + id_tag + operator('in') + expression() +
             keyword('{') + Opt(Lazy(stmt_list)) + keyword('}')
@@ -428,17 +430,16 @@ def foreach_stmt():
 
 def import_stmt():
     def process(p):
-        a, b = p
-        module = b
-        objects = None
-        if isinstance(a, tuple):
-            (((_, module), _), obj), _ = a
-            objects = [obj]
-            for i in b:
-                objects.append(i.value[0])
-        return ImportStmt(module, objects)
+        ((x, module), _), objects = p
+        objects = [i.value[0] for i in objects]
+        from_import = False
+        if isinstance(x, tuple):
+            objects = [module] + objects
+            (_, module), _ = x
+            from_import = True
+        return avo_ast.ImportStmt(module, objects, from_import)
     return Alt(
-        keyword('import') + id_tag,
+        keyword('import') + id_tag + Opt(keyword(',')) + Rep(id_tag + Opt(keyword(','))),
         keyword('from') + id_tag + keyword('import') + id_tag + Opt(keyword(',')) + Rep(id_tag + Opt(keyword(',')))
     ) ^ process
 
@@ -449,11 +450,11 @@ def switch_case_stmt():
         cases_list = []
         for c in cases:
             (((_, cond), _), body), _ = c.value
-            cases_list.append(CaseStmt(cond, body))
+            cases_list.append(avo_ast.CaseStmt(cond, body))
         if else_body:
             ((_, _), else_body), _ = else_body
-            cases_list.append(CaseStmt(None, else_body))
-        return SwitchCaseStmt(var, cases_list)
+            cases_list.append(avo_ast.CaseStmt(None, else_body))
+        return avo_ast.SwitchCaseStmt(var, cases_list)
     return (
             keyword('switch') + expression() + keyword('{') +
             Rep(
@@ -474,7 +475,7 @@ def assign_class_stmt():
             interfaces = [i.value for i in interfaces] + [interface]
         else:
             interfaces = []
-        return AssignClassStmt(name, body, inherit, prefix, interfaces)
+        return avo_ast.AssignClassStmt(name, body, inherit, prefix, interfaces)
     return (
             Opt(keyword('abstract')) + keyword('class') + id_tag + Opt(operator(':') + id_tag) +
             Opt(keyword('of') + id_tag + Rep(id_tag)) +
@@ -485,7 +486,7 @@ def assign_class_stmt():
 def assign_interface_stmt():
     def process(p):
         (((_, name), _), body), _ = p
-        return InterfaceStmt(name, body)
+        return avo_ast.InterfaceStmt(name, body)
     return (
             keyword('interface') + id_tag + keyword('{') + Opt(Lazy(interface_body)) + keyword('}')
     ) ^ process
@@ -493,7 +494,7 @@ def assign_interface_stmt():
 
 def class_body():
     def process(p):
-        return StmtList(p)
+        return avo_ast.StmtList(p)
     return Rep(
         Lazy(class_body_stmt) + Opt(keyword(';')) ^ (lambda x: x[0])
     ) ^ process
@@ -501,7 +502,7 @@ def class_body():
 
 def interface_body():
     def process(p):
-        return StmtList(p)
+        return avo_ast.StmtList(p)
     return Rep(
         Lazy(interface_body_stmt) + Opt(keyword(';')) ^ (lambda x: x[0])
     ) ^ process
@@ -515,10 +516,10 @@ def init_class_stmt():
             (_, args), _ = args
             for arg in args:
                 if arg.value[0][1] is None:
-                    arguments.append(ArgumentAST(arg.value[0][0], None))
+                    arguments.append(avo_ast.ArgumentAST(arg.value[0][0], None))
                 else:
-                    arguments.append(ArgumentAST(arg.value[0][0], arg.value[0][1][1]))
-        return InitClassStmt(arguments, body)
+                    arguments.append(avo_ast.ArgumentAST(arg.value[0][0], arg.value[0][1][1]))
+        return avo_ast.InitClassStmt(arguments, body)
     return (
             keyword('init') + Opt(keyword('(') + Rep(
                id_tag + Opt(operator('=') + Lazy(expression)) + Opt(keyword(','))
@@ -548,7 +549,7 @@ def interface_body_stmt():
 def try_catch_stmt():
     def process(p):
         ((((((_, try_body), _), _), e_name), _), catch_body), _ = p
-        return TryCatchStmt(try_body, e_name, catch_body)
+        return avo_ast.TryCatchStmt(try_body, e_name, catch_body)
     return (
             keyword('try') + keyword('{') + Opt(Lazy(stmt_list)) + keyword('}') +
             keyword('catch') + id_tag + keyword('{') + Opt(Lazy(stmt_list)) + keyword('}')
@@ -577,7 +578,7 @@ def stmt():
             return_stmt() |
             import_stmt() |
             expression() |
-            (Tag(TokenType.EOF) ^ (lambda x: EOFStmt()))
+            (Tag(TokenType.EOF) ^ (lambda x: avo_ast.EOFStmt()))
     )
 
 
